/*
 * DesktopApplication1View.java
 */

package desktopapplication1;

import java.awt.Desktop;
import org.jdesktop.application.Action;
import org.jdesktop.application.ResourceMap;
import org.jdesktop.application.SingleFrameApplication;
import org.jdesktop.application.FrameView;
import org.jdesktop.application.TaskMonitor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.DefaultListModel;
import javax.swing.Timer;
import javax.swing.Icon;
import javax.swing.JDialog;
import javax.swing.JFrame;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.io.*;
import java.net.*;
import java.util.ArrayList;
import java.util.TimerTask;
import java.util.regex.*;


/**
 * The application's main frame.
 */
public class DesktopApplication1View extends FrameView {

    public DesktopApplication1View(SingleFrameApplication app) {
        super(app);

        initComponents();
        initProc();

        // status bar initialization - message timeout, idle icon and busy animation, etc
        ResourceMap resourceMap = getResourceMap();
        int messageTimeout = resourceMap.getInteger("StatusBar.messageTimeout");
        messageTimer = new Timer(messageTimeout, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                statusMessageLabel.setText("");
            }
        });
        messageTimer.setRepeats(false);
        int busyAnimationRate = resourceMap.getInteger("StatusBar.busyAnimationRate");
        for (int i = 0; i < busyIcons.length; i++) {
            busyIcons[i] = resourceMap.getIcon("StatusBar.busyIcons[" + i + "]");
        }
        busyIconTimer = new Timer(busyAnimationRate, new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                busyIconIndex = (busyIconIndex + 1) % busyIcons.length;
                statusAnimationLabel.setIcon(busyIcons[busyIconIndex]);
            }
        });
        idleIcon = resourceMap.getIcon("StatusBar.idleIcon");
        statusAnimationLabel.setIcon(idleIcon);
        progressBar.setVisible(false);

        // connecting action tasks to status bar via TaskMonitor
        TaskMonitor taskMonitor = new TaskMonitor(getApplication().getContext());
        taskMonitor.addPropertyChangeListener(new java.beans.PropertyChangeListener() {
            public void propertyChange(java.beans.PropertyChangeEvent evt) {
                String propertyName = evt.getPropertyName();
                if ("started".equals(propertyName)) {
                    if (!busyIconTimer.isRunning()) {
                        statusAnimationLabel.setIcon(busyIcons[0]);
                        busyIconIndex = 0;
                        busyIconTimer.start();
                    }
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(true);
                } else if ("done".equals(propertyName)) {
                    busyIconTimer.stop();
                    statusAnimationLabel.setIcon(idleIcon);
                    progressBar.setVisible(false);
                    progressBar.setValue(0);
                } else if ("message".equals(propertyName)) {
                    String text = (String)(evt.getNewValue());
                    statusMessageLabel.setText((text == null) ? "" : text);
                    messageTimer.restart();
                } else if ("progress".equals(propertyName)) {
                    int value = (Integer)(evt.getNewValue());
                    progressBar.setVisible(true);
                    progressBar.setIndeterminate(false);
                    progressBar.setValue(value);
                }
            }
        });
    }

    @Action
    public void showAboutBox() {
        if (aboutBox == null) {
            JFrame mainFrame = DesktopApplication1.getApplication().getMainFrame();
            aboutBox = new DesktopApplication1AboutBox(mainFrame);
            aboutBox.setLocationRelativeTo(mainFrame);
        }
        DesktopApplication1.getApplication().show(aboutBox);
    }

    private void initProc(){
        model = new DefaultListModel();
        jList1.setModel(model);
        urls = fileReads(urlfile);
        refreshNets(urls);
        netTimer = new Timer(600000, netRefreshTask);
        netTimer.start();
    }

    ActionListener netRefreshTask = new ActionListener() {
        public void actionPerformed(ActionEvent evt) {
            urls = new ArrayList<String>();
            String x = "";
            try{
                urls = fileReads(urlfile);
                refreshNets(urls);
            }
            catch (Exception e) {
                urls.add("http://www.pheedo.jp/f/slashdot_japan");
                fileWrite(urlfile,x);
            }
        }
    };

    public static void refreshNets(ArrayList<String> urls){
        ArrayList<String> ss = new ArrayList<String>();
        for(int i=0;i<urls.size();i++){
            String seq = getWeb(urls.get(i),"UTF-8");
            if(seq.equals("") == false){
                // for Debug only :|
                // System.out.println(seq);
                fileWrite(tmpfile, seq);
                ss.addAll(getXmlList(tmpfile));
            }
        }
        for(int i=0;i<ss.size();i++){
            model.add(model.size(),ss.get(i));
        }
    }

    	// マッチングルーチン
	static Boolean existStr(String seq,String k){
		pattern = Pattern.compile(k);
		matcher = pattern.matcher(seq);
		return(matcher.matches());
	}

	// マッチングルーチン(Replace)
	static String replaceStr(String seq,String k,String m){
		String r = seq;
		pattern = Pattern.compile(k);
		matcher = pattern.matcher(seq);
		r = matcher.replaceAll(m);
		return(r);
	}

	static void fileWrite(String fn,String seq){
		try {
			FileOutputStream fos = new FileOutputStream(fn);
			OutputStreamWriter osw = new OutputStreamWriter(fos , "UTF-8");
			BufferedWriter bw = new BufferedWriter(osw);
			String msg = seq;
			bw.write(msg);
			bw.close();
			osw.close();
			fos.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

    static ArrayList<String> fileReads(String fn){
        ArrayList<String> s = new ArrayList<String>();
		String seq = "";
		try{
		    File file = new File(fn);
		    BufferedReader br = new BufferedReader(new FileReader(file));

		    String str = br.readLine();
		    while(str != null){
		        //System.out.println(str);
		        seq = seq + str + "\n";
                s.add(str);
		        str = br.readLine();
		    }

		    br.close();
		}catch(FileNotFoundException e){
		    System.out.println(e);
		}catch(IOException e){
		    System.out.println(e);
		}
		return(s);
	}

    static String getXmlValue(Element el,String name){
		String r;
		try{
	        //// name要素のリストを取得
	        //// name要素を取得
	        //// name要素の最初の子ノード（テキストノード）の値を取得
			NodeList descList = el.getElementsByTagName(name);
			Element descElement = (Element)descList.item(0);
			r = descElement.getFirstChild().getNodeValue();
    		r = replaceStr(r,"[\r\n]","");
		}
		catch(Exception e){
			r = "";
		}
		return(r);
	}

	static ArrayList<String> getXmlList(String filename){
		ArrayList<String> s = new ArrayList<String>();
		String r = "";
		try {
	        DocumentBuilderFactory dbfactory = DocumentBuilderFactory.newInstance();
	        DocumentBuilder builder = dbfactory.newDocumentBuilder();
	        Document doc = builder.parse(new File(filename));
	        // ルート要素を取得
	        Element root = doc.getDocumentElement();
	        System.out.println("ルート要素のタグ名：" + root.getTagName());
	        // page要素のリストを取得
	        NodeList list = root.getElementsByTagName("item");
	        System.out.println("要素数: "+list.getLength());
	        // page要素の数だけループ
	        int j=1;
	        for (int i=0; i < list.getLength() ; i++) {
	          // page要素を取得
	            Element element = (Element)list.item(i);
	          // id属性の値を取得
	          //System.out.println(element.getAttribute("title"));
	          //String id = element.getAttribute("id");
	            String title = getXmlValue(element,"title");
	            if(title.equals("") == false && existStr(title,".*AD:.*") == false){
	                // link要素のリストを取得
	            	String link = getXmlValue(element,"link");
	        	    // date要素のリストを取得
	            	String date = getXmlValue(element,"date");
	            	if(date.equals("")){
	            		date = getXmlValue(element,"dc:date");
		            	if(date.equals("")){
		            		date = getXmlValue(element,"date");
		            	}
	            	}
	        	    // description要素のリストを取得
	            	String description = getXmlValue(element,"description");
	            	if(description.equals("")){
	            		description = getXmlValue(element,"dc:desc");
		            	if(description.equals("")){
		            		description = getXmlValue(element,"desc");
		            	}
	            	}
	            	if(description.equals("") == false){
	            		description = replaceStr(description,"\\<.*","");
	            		if(description.length()>128){
	            			description = description.substring(0,120)+"…(つづく)";
	            		}
	            	}
	            	//r = j + ": ";    / for Debug
	            	r = "";
	            	if(date.equals("") == false) r = r + "(" + date + ")";
	            	r = r + "『" + title + "』\n" + link + "\n";
	                if(description.equals("") == false) r = r + description + "\n";
	                s.add(r);
	            	j++;
	            }
	        }
		} catch (Exception e) {
			e.printStackTrace();
		}
		return(s);
	}

	static String getWeb(String url,String ucode){
		String r="";
		try
		{
			URL TestURL = new URL(url);
   	 		URLConnection con = TestURL.openConnection();
			InputStreamReader ir1 = new
			InputStreamReader(con.getInputStream(),ucode);
			BufferedReader br1 = new BufferedReader(ir1);
			String line;
			while((line=br1.readLine()) != null){
				r = r + line + "\n";
			}
			br1.close();
			ir1.close();
		}
		catch(Exception e)
		{
			e.printStackTrace();
			r = "";
		}
		return(r);
	}
    
    static void openUrl(String url){
        Desktop desktop = Desktop.getDesktop();
        try {
			desktop.browse(new URI(url));
		} catch (IOException e1) {
			// TODO 自動生成された catch ブロック
			e1.printStackTrace();
		} catch (URISyntaxException e1) {
			// TODO 自動生成された catch ブロック
			e1.printStackTrace();
		}
	}

	static String getUrl(String seq){
		String r = "";
		int i = seq.indexOf("http://");
		r = seq.substring(i,seq.length());
		r = r.substring(0,r.indexOf("\n"));
		return(r);
	}


    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        mainPanel = new javax.swing.JPanel();
        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        jList1 = new javax.swing.JList();
        menuBar = new javax.swing.JMenuBar();
        javax.swing.JMenu fileMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem exitMenuItem = new javax.swing.JMenuItem();
        javax.swing.JMenu helpMenu = new javax.swing.JMenu();
        javax.swing.JMenuItem aboutMenuItem = new javax.swing.JMenuItem();
        statusPanel = new javax.swing.JPanel();
        javax.swing.JSeparator statusPanelSeparator = new javax.swing.JSeparator();
        statusMessageLabel = new javax.swing.JLabel();
        statusAnimationLabel = new javax.swing.JLabel();
        progressBar = new javax.swing.JProgressBar();

        mainPanel.setName("mainPanel"); // NOI18N

        jPanel1.setName("jPanel1"); // NOI18N

        jScrollPane1.setName("jScrollPane1"); // NOI18N

        jList1.setName("jList1"); // NOI18N
        jList1.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jList1MouseClicked(evt);
            }
        });
        jList1.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyTyped(java.awt.event.KeyEvent evt) {
                jList1KeyTyped(evt);
            }
        });
        jScrollPane1.setViewportView(jList1);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 248, Short.MAX_VALUE)
        );

        javax.swing.GroupLayout mainPanelLayout = new javax.swing.GroupLayout(mainPanel);
        mainPanel.setLayout(mainPanelLayout);
        mainPanelLayout.setHorizontalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        mainPanelLayout.setVerticalGroup(
            mainPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        menuBar.setName("menuBar"); // NOI18N

        org.jdesktop.application.ResourceMap resourceMap = org.jdesktop.application.Application.getInstance(desktopapplication1.DesktopApplication1.class).getContext().getResourceMap(DesktopApplication1View.class);
        fileMenu.setText(resourceMap.getString("fileMenu.text")); // NOI18N
        fileMenu.setName("fileMenu"); // NOI18N

        javax.swing.ActionMap actionMap = org.jdesktop.application.Application.getInstance(desktopapplication1.DesktopApplication1.class).getContext().getActionMap(DesktopApplication1View.class, this);
        exitMenuItem.setAction(actionMap.get("quit")); // NOI18N
        exitMenuItem.setName("exitMenuItem"); // NOI18N
        fileMenu.add(exitMenuItem);

        menuBar.add(fileMenu);

        helpMenu.setText(resourceMap.getString("helpMenu.text")); // NOI18N
        helpMenu.setName("helpMenu"); // NOI18N

        aboutMenuItem.setAction(actionMap.get("showAboutBox")); // NOI18N
        aboutMenuItem.setName("aboutMenuItem"); // NOI18N
        helpMenu.add(aboutMenuItem);

        menuBar.add(helpMenu);

        statusPanel.setName("statusPanel"); // NOI18N

        statusPanelSeparator.setName("statusPanelSeparator"); // NOI18N

        statusMessageLabel.setName("statusMessageLabel"); // NOI18N

        statusAnimationLabel.setHorizontalAlignment(javax.swing.SwingConstants.LEFT);
        statusAnimationLabel.setName("statusAnimationLabel"); // NOI18N

        progressBar.setName("progressBar"); // NOI18N

        javax.swing.GroupLayout statusPanelLayout = new javax.swing.GroupLayout(statusPanel);
        statusPanel.setLayout(statusPanelLayout);
        statusPanelLayout.setHorizontalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(statusPanelSeparator, javax.swing.GroupLayout.DEFAULT_SIZE, 400, Short.MAX_VALUE)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(statusMessageLabel)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 218, Short.MAX_VALUE)
                .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(statusAnimationLabel)
                .addContainerGap())
        );
        statusPanelLayout.setVerticalGroup(
            statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(statusPanelLayout.createSequentialGroup()
                .addComponent(statusPanelSeparator, javax.swing.GroupLayout.PREFERRED_SIZE, 2, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                .addGroup(statusPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(statusMessageLabel)
                    .addComponent(statusAnimationLabel)
                    .addComponent(progressBar, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(3, 3, 3))
        );

        setComponent(mainPanel);
        setMenuBar(menuBar);
        setStatusBar(statusPanel);
    }// </editor-fold>//GEN-END:initComponents

    private void jList1MouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jList1MouseClicked
				if(evt.getClickCount() == 2){
					openUrl(getUrl((String)jList1.getSelectedValue()));
				}
    }//GEN-LAST:event_jList1MouseClicked

    private void jList1KeyTyped(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_jList1KeyTyped
        openUrl(getUrl((String)jList1.getSelectedValue()));
    }//GEN-LAST:event_jList1KeyTyped

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JList jList1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JPanel mainPanel;
    private javax.swing.JMenuBar menuBar;
    private javax.swing.JProgressBar progressBar;
    private javax.swing.JLabel statusAnimationLabel;
    private javax.swing.JLabel statusMessageLabel;
    private javax.swing.JPanel statusPanel;
    // End of variables declaration//GEN-END:variables

    private static String frameTitle = "RSSsample ACT2";
    private final Timer messageTimer;
    private final Timer busyIconTimer;
    private static Timer netTimer;
    private final Icon idleIcon;
    private final Icon[] busyIcons = new Icon[15];
    private int busyIconIndex = 0;
    private static DefaultListModel model;
    private JDialog aboutBox;
    private static Pattern pattern;
	private static Matcher matcher;
    private static String urlfile = "urls.txt";
    private static String tmpfile = "9765432.9";
    private static ArrayList<String> urls;

}
